#!/bin/sh
NAME=scriptserver
APKG_PATH=/usr/local/AppCentral/${NAME}
FOLDER="${APKG_PATH}/script-server"
MY_DATA=${APKG_PATH}/my_data
HTPASSWD_FOLDER=${APKG_PATH}/my_data
JSON_ADMIN=${MY_DATA}/runners
FOLDER_TO_LINK=""
TEMP_INSTALL=${APKG_PATH}/temp_install
BLUE="\033[0;36m"
NC="\033[0m"
RED="\033[0;31m"
PROG=${MY_DATA}/htpasswd

create_link_for_folder(){
	if [ -e ${MY_DATA}/folder.conf ] ; then
	  while IFS= read line
	  do
		if [ "${line:0:1}" = "#" ] ; then
			continue
		elif [ -z "${line}" ] ; then
			continue
		else
			FOLDER_NAME="$(echo $line | cut -f 1 -d ':')"
			if [ ! -z "${FOLDER_TO_LINK}" ] ; then
				if [ "${FOLDER_NAME}" != "${FOLDER_TO_LINK}" ] ; then
					continue
				fi
			fi
			
			PATH_TO_FOLDER="$(echo $line | cut -f 3 -d ':')"
			if [ ! -e ${PATH_TO_FOLDER} ] ; then
				echo "HUM! FOLDER ${FOLDER_NAME} seem to don't exist ???"
				continue
			fi
			
			if [ -e ${PATH_TO_FOLDER}/script_server ] ; then
				SC_USER="$(echo $line | cut -f 2 -d ':')"
				if [ ! -z "${SC_USER}" ] ; then
					grep -q "^${SC_USER}:" ${MY_DATA}/.htpassword
					if [ $? = 0 ] ; then
						echo "User ${SC_USER} is already defined"
					else
						echo "Need to create a new use : ${SC_USER} with asusfr1 as default password CHANGE IT"
						$0 add_new_user_password ${SC_USER} asusfr1 asusfr1
					fi
				fi
				
				CUR_PWD="$(pwd)"
				cd ${FOLDER}/conf/runners
				echo "time to create link from Script-Server to ${FOLDER_NAME} json files"
				for F in ${PATH_TO_FOLDER}/script_server/*.json
				do
					if [ -e "$(basename ${F})" ] ; then
						rm "$(basename ${F})"
					fi
					ln -s "${F}"
				done
				cd "${CUR_PWD}"
				
				echo -e "${BLUE}Done for ${FOLDER_NAME} ${NC}"
			else
				echo -e "${RED}NO script-server folder for ${FOLDER_NAME} ${NC}"
			fi
		fi
	  done < ${MY_DATA}/folder.conf
	fi
}

list_folder_defined(){
	if [ -e ${MY_DATA}/folder.conf ] ; then
	  echo "LIST of folder defined in folder.conf"
	  while IFS= read line
	  do
		if [ "${line:0:1}" = "#" ] ; then
			echo "Comment : ${line}"
		elif [ -z "${line}" ] ; then
			continue
		else
			FOLDER_NAME="$(echo $line | cut -f 1 -d ':')"
			SC_USER="$(echo $line | cut -f 2 -d ':')"
			if [ -z "${SC_USER}" ] ; then
				SC_USER="ALL"
			fi
			PATH_TO_FOLDER="$(echo $line | cut -f 3 -d ':')"
			echo ""
			echo -e "${BLUE}${FOLDER_NAME}${NC} ... is for : ${BLUE}${SC_USER}${NC} (script_server user) ... script are in : ${BLUE}${PATH_TO_FOLDER}${NC} (in script_server folder inside)"
			# echo "${FOLDER_NAME} Associated to script-server USER : ${SC_USER} ... PATH : ${PATH_TO_FOLDER}/script_server "
		fi
	  done < ${MY_DATA}/folder.conf
	fi
}

create_new_folder(){
	echo "${FOLDER_NAME}:${SC_USER}:${PATH_TO_FOLDER}" >> ${MY_DATA}/folder.conf
}

delete_json_from_runners(){
	# PATH_TO_FOLDER is provide
	echo "Start deleting JSON link from ${FOLDER}/conf/runners "
	CUR_PWD=$(pwd)
	cd ${FOLDER}/conf/runners
	for F in ./*.json
	do 
		P="$(readlink $F | grep ${PATH_TO_FOLDER})"
		if [ ! -z "${P}" ] ; then
			rm -f "${F}"
		fi 
	done
	cd "${CUR_PWD}"
	echo "Done"
}

check_runners_link(){
	for F in ${APKG_PATH}/script-server/conf/runners/*
	do	# ONLY for link 
		if [ -L "${F}" ] ; then
			if [ ! -e "${F}" ] ; then
				rm -f "${F}"
			fi
		fi
	done
}

delete_folder(){
	## first delete json
	# get folder path
	echo -e "DELETE FOLDER definition for ${FOLDER_NAME} in ${MY_DATA}/folder.conf"
	PATH_TO_FOLDER=$(grep "^${FOLDER_NAME}:" ${MY_DATA}/folder.conf | cut -f 3 -d ':')
	delete_json_from_runners
	# delete in folder.conf
	echo -e "NOW delete folder definition"
	sed -i "/^${FOLDER_NAME}:/d" ${MY_DATA}/folder.conf
	echo -e "Done"
	#
}

link_admin_json(){
	CUR_PWD=$(pwd)
	cd ${FOLDER}/conf/runners
	for F in ${JSON_ADMIN}/*
	do 
		if [ -e "$(basename ${F})" ] ; then
			rm "$(basename ${F})"
		fi
		ln -s "${F}"
	done
	cd "${CUR_PWD}"
}

old_test_version(){
			V_MAJOR=$(echo ${V} | cut -f 1 -d ".")
			if [ -z "${V_MAJOR}" ] ; then
				V_MAJOR=0
			fi
			V_MINOR=$(echo ${V} | cut -f 2 -d ".")
			if [ -z "${V_MINOR}" ] ; then
				V_MINOR=0
			fi
			V_LAST=$(echo ${V} | cut -f 3 -d ".")
			if [ -z "${V_LAST}" ] ; then
				V_LAST=0
			fi
			PREV_MAJOR=$(echo ${PREV} | cut -f 1 -d ".")
			if [ -z "${PREV_MAJOR}" ] ; then
				PREV_MAJOR=0
			fi
			PREV_MINOR=$(echo ${PREV} | cut -f 2 -d ".")
			if [ -z "${PREV_MINOR}" ] ; then
				PREV_MINOR=0
			fi
			PREV_LAST=$(echo ${PREV} | cut -f 3 -d ".")
			if [ -z "${PREV_LAST}" ] ; then
				PREV_LAST=0
			fi
			# REJECT="FALSE"
			if [ ${PREV_MAJOR} -gt ${V_MAJOR} ] ; then
				echo -e "version of scripts provide ( ${V} ) is lower that actual installed version"
				rm -f "${TEMP_INSTALL}/"*
				exit 1
			elif [ ${PREV_MINOR} -gt ${V_MINOR} ] ; then
				if [ ${PREV_MINOR} -gt ${V_MINOR} ] ; then
					echo -e "version of scripts provide ( ${V} ) is lower that actual installed version"
					rm -f "${TEMP_INSTALL}/"*
					exit 1
				fi
			elif [ ${PREV_LAST} -gt ${V_LAST} ] ; then
				echo -e "version of scripts provide ( ${V} ) is lower that actual installed version"
				rm -f "${TEMP_INSTALL}/"*
				exit 1
			fi
}

integrate_apkg_folder(){
	if [ ! -e "${TEMP_INSTALL}" ] ; then 
		mkdir "${TEMP_INSTALL}"
	fi
	# CUR_PWD=$(pwd)
	/bin/tar xzf "${PATH_INTEG}" -C "${TEMP_INSTALL}"
	if [ ! -e "${TEMP_INSTALL}/new_folder.conf" ] ; then
		echo -e "Your file ${RED}$(basename ${PATH_INTEG})${NC} don't have a new_folder.conf inside"
		exit 1
	else	
		T=$(/usr/bin/confutil -get "${TEMP_INSTALL}/new_folder.conf" script_server type)
		if [ $? -ne 0 ] ; then
			echo -e "HUM!! error the type is not defined"
			exit 1
		else
			if [ "${T}" = "APKG" ] ; then
				APKG=$(/usr/bin/confutil -get "${TEMP_INSTALL}/new_folder.conf" script_server apkg_name)
				if [ $? -ne 0 ] ; then
					echo -e "HUM!! error the type is incorrect"
					exit 1
				else
					PA=/usr/local/AppCentral/"${APKG}"
					if [ ! -e "${PA}" ] ; then
						echo -e "Can't access the folder define ? : ${PA} don't exist"
						exit 1
					fi
				fi
			elif [ "${T}" = "FOLDER" ] ; then
				PA=$(/usr/bin/confutil -get "${TEMP_INSTALL}/new_folder.conf" script_server path_folder)
				if [ $? -ne 0 ] ; then
					echo -e "HUM!! error the type is incorrect"
					exit 1
				fi
				if [ ! -e "${PA}" ] ; then
					mkdir -p "${PA}"
				fi
			else
				echo -e "HUM!! error the type is incorrect"
				exit 1
			fi
		fi
		# cp -pP "${TEMP_INSTALL}"/* "${PA}"/
		FOLDER_NAME=$(/usr/bin/confutil -get "${TEMP_INSTALL}/new_folder.conf" script_server folder_name)
		if [ $? -ne 0 ] ; then
			echo -e "YOU MUST provide a folder_name ???"
			exit 1
		fi
		
		V=$(/usr/bin/confutil -get "${TEMP_INSTALL}/new_folder.conf" script_server version)
		if [ $? -ne 0 ] ; then
			echo -e "version MUST be provide form: 1.2 or 1.2.3"
			exit 1
		fi
		
		if [ ! -e "${PA}"/script_server ] ; then
			mkdir "${PA}"/script_server
		else
			PREV=$(/usr/bin/confutil -get "${PA}/script_server/new_folder.conf" script_server version)
			if [ $? -ne 0 ] ; then
				### strange
				echo -e "Hum script_server folder exist but is not conform ? no new_folder.conf ???"
				PREV="${V}" # to get around missing . conf
			fi
			### now compare version
#			old_test_version
			if [ "${V}" = "${PREV}" ] ; then
				echo -e "New version ${V} same as previous version ${PREV} Nothing to do"
				exit 1
			else 
				# simple test todo changing the old test
				LT_VERSION=$(echo -e "${V}\n${PREV}" | sort -t "." -k 1,1 -k 2,2 -k 3,3 -k 4,4 -k 5,5 -k 6,6 |head -n 1)
				if [ "${V}" = "${LT_VERSION}" ] ; then
					echo -e "HUM! new version ${V} is less than actual version ${PREV} "
					exit 1
				fi
			fi
			###
		fi
		cp -pP "${TEMP_INSTALL}/"* "${PA}"/script_server/
		# now we can change TEMP_INSTALL by PA/script_server
		U=$(/usr/bin/confutil -get "${PA}/script_server/new_folder.conf" script_server user)
		if [ $? -ne 0 ] ; then
			echo -e "a user MUST be provide (or htpassword or NONE)"
			exit 1
		fi
		grep -q "^${U}:" ${MY_DATA}/.htpassword
		if [ $? = 0 ] ; then
			echo -e "User already define ... continue to create or update scripts"
		else
			if [ "${U}" = "htpassword" ] ; then
				RU=$(cut -f 1 -d ":" < "${PA}"/script_server/htpassword)
				grep -q "^${RU}:" ${MY_DATA}/.htpassword
				if [ $? = 0 ] ; then
					echo -e "User already defined ... continue to create or update scripts"
				else
					echo -e "ADD user define in htpasword file provide by tgz file with already encrypted password"
					cat "${PA}"/script_server/htpassword >> ${MY_DATA}/.htpassword
					rm -f "${PA}"/script_server/htpassword # for security
				fi
				U="${RU}"
			else
				if [ "${U}" = "NONE" ] ; then
					echo "All user can run this scripts except if the script force a user"
					U=""
				else
					echo -e "ADD new user with name provide using default password asusfr1"
					$0 add_new_user_password "${U}" asusfr1 asusfr1
					echo "Done"
				fi
			fi
		fi

		## NOW it's time to ad folder in folder.conf 
		echo -e "ADD definition in folder.conf"
		
		## test if FOLDER_NAME is already define 
		grep -q "^${FOLDER_NAME}:" ${MY_DATA}/folder.conf
		if [ $? -eq 0 ] ; then
			echo -e "${FOLDER_NAME} already exist in ${MY_DATA}/folder.conf"
		else
			echo "${FOLDER_NAME}:${U}:${PA}" >> ${MY_DATA}/folder.conf
		fi 
		echo -e "Create link in script_server"
		$0 create_folder_link "${FOLDER_NAME}"
		echo -e "ALL is now done ... now cleaning temp files"
		rm -f "${TEMP_INSTALL}/"*
	fi
}

save_settings(){
	FILE_NAME="script_server_$(date +%m_%d_%Y_%Hh%Mm%S)"
	echo "Saving script_server settings (root user) ... conf folder in /share/Public/.${NAME}/${FILE_NAME}"
	if [ ! -e /share/Public/.${NAME} ] ; then
		mkdir /share/Public/.${NAME}
	fi
	if [ ! -e /share/Public/.${NAME}/${FILE_NAME} ] ; then
		mkdir /share/Public/.${NAME}/${FILE_NAME}
	else
		echo "Hum! folder already exist ???? exit"
		exit 1
	fi
	cp -pPR ${APKG_PATH}/script-server/conf /share/Public/.${NAME}/${FILE_NAME}	
	echo "Save settings done in /share/Public/.${NAME}/${FILE_NAME}"
	echo "PLEASE think to delete old and unused save (folder in /share/Public/.${NAME}/)"
}

save_all_logs(){
	FILE_NAME="script_server_logs_$(date +%m_%d_%Y_%Hh%Mm%S)"
	echo "Saving script_server LOGS ... logs folder in /share/Public/.${NAME}/${FILE_NAME}"
	if [ ! -e /share/Public/.${NAME} ] ; then
		mkdir /share/Public/.${NAME}
	fi
	if [ ! -e /share/Public/.${NAME}/${FILE_NAME} ] ; then
		mkdir /share/Public/.${NAME}/${FILE_NAME}
	else
		echo "Hum! folder already exist ???? exit"
		exit 1
	fi
	cp -pPR ${APKG_PATH}/script-server/logs /share/Public/.${NAME}/${FILE_NAME}	
	echo "Save logs done in /share/Public/.${NAME}/${FILE_NAME}"
	echo "PLEASE think to delete old and unused save (folder in /share/Public/.${NAME}/)"
}

add_new_user_password(){
	if [ -z "${U}" ] ; then
		echo "PLEASE provide a user name"
		exit 1
	else
		grep -q "^${U}:" ${HTPASSWD_FOLDER}/.htpassword
		if [ $? -eq 0 ] ; then
			echo "HUM! user already exist ... use change_user_password if you only want to change password"
			exit 1
		else
			if [ -z "${P}" ] ; then
				echo "Please enter password when required "
				$PROG ${HTPASSWD_FOLDER}/.htpassword "${U}"
			else
				if [ "${P}" != "${CP}" ] ; then
					echo "Password and Confirm password don't match"
					exit 1
				else
					echo "setting new password to new user ${U}"
					$PROG -b ${HTPASSWD_FOLDER}/.htpassword "${U}" "${P}"
				fi
			fi
			echo -e "User and interactive or batch password created"
			# $0 restart
		fi
	fi
}

supress_user_password(){
	grep -q "^${U}:" ${HTPASSWD_FOLDER}/.htpassword
	if [ $? -eq 0 ] ; then
		read -p "Please enter Y (uppercase) then ENTER to confirm or not or any key(s) to cancel : " XX
		XX=${XX:0:1}
		echo ""
		if [ "$XX" = "Y" ] ; then
			$PROG -D ${HTPASSWD_FOLDER}/.htpassword ${U}
			echo -e "User & Password have been removed"
		else
			echo "YOU HAVE CANCELED the suppression"
		fi
	else
		echo "user ${U} don't exist in security access file"
		exit 1
	fi
}

remove_history_logs(){
	if [ -z "${U}" ] ; then
		echo "Ready to remove ALL history logs ? "
	else
		grep -q "^${U}:" ${HTPASSWD_FOLDER}/.htpassword
		if [ $? -eq 0 ] ; then
			echo "Ready to remove USER : ${U} history logs ? "
		else
			echo "Hum! user don't exist"
			exit 0
		fi
	fi
	read -p "Please enter Y (uppercase) then ENTER to confirm or no or any key(s) to cancel : " XX
	XX=${XX:0:1}
	echo ""
	if [ "$XX" = "Y" ] ; then
		if [ -z "${U}" ] ; then
			rm -f ${APKG_PATH}/script-server/logs/processes/*.log
		else
			rm -f ${APKG_PATH}/script-server/logs/processes/*_${U}_*.log
		fi
	fi
	echo "=== done"
}

change_user_password(){
	if [ -z "${U}" ] ; then
		echo "PLEASE provide a user name"
		exit 1
	else
		grep -q "^${U}:" ${HTPASSWD_FOLDER}/.htpassword
		if [ $? -ne 0 ] ; then
			echo "HUM! user don't exist ... use add_new_user_password if you only want to add a new user/password"
			exit 1
		else
			if [ -z "${P}" ] ; then
				echo "Please enter password when required "
				$PROG ${HTPASSWD_FOLDER}/.htpassword "${U}"
			else
				if [ "${P}" != "${CP}" ] ; then
					echo "Password and Confirm password don't match"
					exit 1
				else
					echo "setting new password to user ${U}"
					$PROG -b ${HTPASSWD_FOLDER}/.htpassword "${U}" "${P}"
				fi
			fi
			$0 restart
		fi
	fi
}

verif_user_password(){
	if [ -z "${U}" ] ; then
		echo "PLEASE provide a user name"
		exit 1
	else
		grep -q "^${U}:" ${HTPASSWD_FOLDER}/.htpassword
		if [ $? -ne 0 ] ; then
			echo "HUM! user don't exist ... you can't verify password ????"
			exit 1
		else
			if [ -z "${P}" ] ; then
				echo "Please enter actual password when required "
				$PROG -v ${HTPASSWD_FOLDER}/.htpassword "${U}"
			else
				$PROG -bv ${HTPASSWD_FOLDER}/.htpassword "${U}" "${P}"
			fi
		fi
	fi
}

python_venv(){
if [ ! -e ${APKG_PATH}/python3_venv ] ; then
	rm -f /tmp/sc_python_venv_modules.log
	CURPWD="$(pwd)"
	cd ${APKG_PATH}
# create the virtualenv
	/usr/local/bin/python3 -m venv ${APKG_PATH}/python3_venv
# enter in the virtualenv
	source ${APKG_PATH}/python3_venv/bin/activate
# Upgrade pip is needed
	pip install --upgrade pip
# upgrade or install wheel
	pip install -U wheel
# Install requirement
	pip install -r ${APKG_PATH}/script-server/requirements.txt
# install ldap and bcrypt in case of used
	pip install ldap3
	pip install bcrypt
# NOW all is ready to used by script-server
# before deactivate virtualenv, list modules for info
	pip list > /tmp/sc_python_venv_modules.log
	deactivate
	cd "${CURPWD}"
fi
}

run_once(){
	if [ ! -e ${FOLDER}/logs ] ; then
		mkdir ${FOLDER}/logs
	fi
	if [ ! -e /usr/local/bin/script_server_mngt ] ; then
		ln -s ${APKG_PATH}/CONTROL/start-stop.sh /usr/local/bin/script_server_mngt
	fi
	if [ ! -e ${MY_DATA}/.htpassword ] ; then
		mv ${MY_DATA}/.htpassword.delivery ${MY_DATA}/.htpassword 
	else
		if [ -e ${MY_DATA}/.htpassword.delivery ] ; then
			rm -f ${MY_DATA}/.htpassword.delivery
		fi
	fi
	if [ ! -e ${FOLDER}/conf/script-server.json ] ; then
		cp -p ${MY_DATA}/script-server.json ${FOLDER}/conf/script-server.json
	fi
	link_admin_json
	if [ ! -e ${MY_DATA}/folder.conf ] ; then
		mv ${MY_DATA}/folder.conf.delivery ${MY_DATA}/folder.conf
	else
		if [ -e ${MY_DATA}/folder.conf.delivery ] ; then
			rm -f ${MY_DATA}/folder.conf.delivery
		fi
	fi
	create_link_for_folder
	touch /tmp/.scriptserver.lock
} 

if [ -e ${MY_DATA}/.use_my_htpasswd ] ; then
	rm -f ${MY_DATA}/htpasswd
	ln -s ${MY_DATA}/htpasswd.$(uname -m) ${MY_DATA}/htpasswd
else
	rm -f ${MY_DATA}/htpasswd
	if [ -e "/volume1/.@plugins/AppCentral/httpd-2.4.43/data/binary/bin/htpasswd" ] ; then
		ln -s "/volume1/.@plugins/AppCentral/httpd-2.4.43/data/binary/bin/htpasswd" ${MY_DATA}/htpasswd
	else
		ln -s ${MY_DATA}/htpasswd.$(uname -m) ${MY_DATA}/htpasswd
	fi
fi


case "$1" in
start)
	if [ ! -e /tmp/.scriptserver.lock ] ; then
		run_once
	fi
	python_venv
	echo -e "Start Script-server"
	if [ -e ${FOLDER}/logs/launcher.log.prev ] ; then
		rm -f ${FOLDER}/logs/launcher.log.prev
	fi
	if [ -e ${FOLDER}/logs/launcher.log ] ; then
		mv ${FOLDER}/logs/launcher.log ${FOLDER}/logs/launcher.log.prev
	fi
	for N in $(seq 5 -1 1)
	do
		if [ -e ${FOLDER}/logs/server.log.${N} ] ; then
			if [ ${N} -eq 5 ] ; then
				rm -f ${FOLDER}/logs/server.log.${N}
			else
				M=$((N + 1))
				mv ${FOLDER}/logs/server.log.${N} ${FOLDER}/logs/server.log.${M}
			fi
		fi
	done
	if [ -e ${FOLDER}/logs/server.log ] ; then
		mv ${FOLDER}/logs/server.log ${FOLDER}/logs/server.log.1
	fi
	
	### check loop link broken
	check_runners_link
	
	cd ${FOLDER}
	source ${APKG_PATH}/python3_venv/bin/activate
	hash -r
	PYT=$(which python3)
	# echo "${PYT}"
	nohup "${PYT}" launcher.py -d ./conf -f script-server.json >${FOLDER}/logs/launcher.log 2>&1  &
	echo $! > ./launcher.pid
	echo -e "Start is done"
;;
stop)
	echo -e "Sopt Script-server"
	if [ -e ${FOLDER}/launcher.pid ] ; then
		kill -9 "$(cat ${FOLDER}/launcher.pid)"
		rm -f ${FOLDER}/launcher.pid
	fi
	sleep 1
	### in case of bad PID
	PID=$(ps -eaf | grep -v grep | grep launcher.py | tr -s " " | cut -f 2 -d " ")
	if [ ! -z "$PID" ] ; then
		kill -9 "${PID}"
	fi
	# rm -f /tmp/.scriptserver.lock
	echo -e "Stop done"
;;
restart)
	$0 stop 
	sleep 1
	rm -f /tmp/.scriptserver.lock
	$0 start 
;;
clean_and_restart)
	$0 stop 
	sleep 1
	rm -f ${FOLDER}/conf/runners/*.json
	rm -f /tmp/.scriptserver.lock
	$0 start 
;;
rebuild_venv)
	$0 stop
	sleep 1
	if [ -e ${APKG_PATH}/python3_venv.prev ] ; then
		rm -rf ${APKG_PATH}/python3_venv.prev
	fi
	mv ${APKG_PATH}/python3_venv ${APKG_PATH}/python3_venv.prev
	## start rebuil venv ...
	$0 start
;;
status)
	echo -e "### search for script-server (aka launcher.py) ... no line, script-server is stopped"
	ps -eaf | grep -v grep | grep "/usr/local/AppCentral/scriptserver" | grep launcher.py
	echo "====="
	echo -e "### htpasswd come from Apache Asustor Web server OR from this APKG ? :"
	/usr/bin/readlink ${MY_DATA}/htpasswd
	echo -e "====="
;;
relink_admin_json)
	link_admin_json
;;
integrate_apkg_folder)
	PATH_INTEG="$2"
	if [ ! -e "${PATH_INTEG}" ] ; then
		echo -e "Please provide a valid tar gzip (file.tgz) full Path"
		exit 1
	fi
	integrate_apkg_folder
;;
webui_port_used)
	echo "==== Port used by server (even not running) ===="
	PORT=$(grep \"port\": ${FOLDER}/conf/script-server.json |tr -d " " | cut -f 2 -d ":" | cut -f 1 -d ",")
	echo -e "Port : ${BLUE}${PORT}${NC} for server"
	echo "===="
;;
save_settings)
	save_settings
;;
save_all_logs)
	save_all_logs
;;
remove_history_logs)
	U="${2}"
	remove_history_logs
;;
list_users_defined)
	echo "==== list of users with access to script_server"
	echo "==== remember script_admin user can't be suppressed"
	for U in $(cut -f 1 -d ":" < ${HTPASSWD_FOLDER}/.htpassword)
    do
        if [ "$U" = "script_admin" ] ; then
           echo -e "${RED}${U}${NC}"
        else
           echo -e "${BLUE}${U}${NC}"
        fi
    done
	echo "===="
;;
supress_user_password)
	U="${2}"
	if [ "${U}" = "script_admin" ] ; then
		echo "==== script_admin can't be deleted, but password associated CAN/MUST (if initial) be changed"
		exit 1
	fi
	supress_user_password	
;;
add_new_user_password)
	U="${2}"
	P="${3}"
	CP="${4}"
	add_new_user_password
	
;;
change_user_password)
	U="${2}"
	P="${3}"
	CP="${4}"
	change_user_password
;;
verif_user_password)
	U="${2}"
	P="${3}"
	verif_user_password
;;
change_current_user_password)
	XUSER_NAME="${2}"
	XACTUAL_P="${3}"
	XNEWP="${4}"
	XCONFIRMP="${5}"
	$0 verif_user_password "${XUSER_NAME}" "${XACTUAL_P}"
	if [ $? -eq 0 ] ; then
		$0 change_user_password "${XUSER_NAME}" "${XNEWP}" "${XCONFIRMP}"
	fi
;;
change_to_https)
	### it's an hidden function not documented not listed in help 
	grep -q "\"ssl\":" ${FOLDER}/conf/script-server.json
	if [ $? -eq 1 ] ; then
		if [ -e ${FOLDER}/conf/add_ssl_script-server.json ] ; then
			sed -i "/\"title\":/r ${FOLDER}/conf/add_ssl_script-server.json" ${FOLDER}/conf/script-server.json
			echo "ADD (without verification) ssl lines from ${FOLDER}/conf/add_ssl_script-server.json"
		else
			echo -e "PLEASE stop script-server then add SSL section in ${FOLDER}/conf/script-server.json then start script-server"
			echo -e "${RED}BEFORE${NC} using this option"
			exit 1
		fi
	fi
	FILE="/usr/builtin/etc/appcentral/app-config.json"
	if [ ! -e ${FILE}.ori ] ; then
		cp -p ${FILE} ${FILE}.ori
	fi
	PORT=$(grep \"port\": ${FOLDER}/conf/script-server.json |tr -d " " | cut -f 2 -d ":" | cut -f 1 -d ",")
	# get line with port used then -1 for protocol
	N=$(grep -n ${PORT} ${FILE} | cut -f 1 -d ":")
	N=$(( N -1 ))
	if [ "$(sed "${N}q;d" ${FILE} | tr -d " " | cut -f 2 -d ':' | tr -d '"' | tr -d ',')" = "https" ] ; then
		echo "NOTHING to do already https"
	else
		echo -e "Change http to https ... no Port change"
		sed -i "${N}s/http\"/https\"/" ${FILE}
	fi
	$0 restart
;;
change_to_http)
	### it's an hidden function not documented not listed in help
	grep -q "\"ssl\":" ${FOLDER}/conf/script-server.json
	if [ $? -eq 0 ] ; then
		echo "Suppress ssl paragraph in ${FOLDER}/conf/script-server.json "
		sed -i "/\"ssl\":/,+3d" ${FOLDER}/conf/script-server.json
	else
		echo -e "${RED}NO${NC} ssl section in script-server.json ... so ???"
		exit 1
	fi
	FILE="/usr/builtin/etc/appcentral/app-config.json"
	if [ ! -e ${FILE}.ori ] ; then
		cp -p ${FILE} ${FILE}.ori
	fi
	PORT=$(grep \"port\": ${FOLDER}/conf/script-server.json |tr -d " " | cut -f 2 -d ":" | cut -f 1 -d ",")
	# get line with port used then -1 for protocol
	N=$(grep -n ${PORT} ${FILE} | cut -f 1 -d ":")
	N=$(( N -1 ))
	if [ "$(sed "${N}q;d" ${FILE} | tr -d " " | cut -f 2 -d ':' | tr -d '"' | tr -d ',')" = "http" ] ; then
		echo "NOTHING to do already http"
	else
		echo -e "Change https to http ... no Port change"
		sed -i "${N}s/https\"/http\"/" ${FILE}
	fi
	$0 restart
;;
change_webui_port_used)
	NEW_PORT="${2}"
	if [ -z "${NEW_PORT}" ] ; then
		echo -e "Port for WebUI is not provide"
		exit 1
	fi
	FILE="/usr/builtin/etc/appcentral/app-config.json"
	if [ ! -e ${FILE}.ori ] ; then
		cp -p ${FILE} ${FILE}.ori
	fi
	grep -q ${NEW_PORT} ${FILE}
	if [ $? -eq 0 ] ; then
		echo "Hum new port seem to exist in some APKG ??? ... exit"
		exit 1
	else
		N=$(grep -n ${PORT} ${FILE} | cut -f 1 -d ":")
		PORT=$(grep \"port\": ${FOLDER}/conf/script-server.json |tr -d " " | cut -f 2 -d ":" | cut -f 1 -d ",")
		sed -i "${N}s/${PORT}/${NEW_PORT}/" ${FILE}
		echo -e "Port change from : ${PORT} to new value ${NEW_PORT} ... check that /usr/local/AppCentral/scriptserver/script-server/conf/script-server have the new value"
	fi
	$0 restart
;;
create_folder_link)
	if [ ! -z "${2}" ] ; then
		FOLDER_TO_LINK="${2}"
	else
		FOLDER_TO_LINK=""
	fi
	create_link_for_folder
	echo "Done"
;;
list_folder_defined)
	list_folder_defined
;;
create_new_folder)
	FOLDER_NAME="${2}"
	PATH_TO_FOLDER="${3}"
	SC_USER="${4}"
	P="${5}"
	PC="${6}"
	if  [ -z "${FOLDER_NAME}" ] ; then
		echo "missing parameters folder_name"
		exit 1
	fi
	if [ -z "${SC_USER}" ] ; then
		echo "missing parameter user owning script use NONE for all users"
		exit 1
	fi
	if [ -z "${PATH_TO_FOLDER}" ] ; then
		echo "missing parameter path to folder script"
		exit 1
	fi
	if [ "${SC_USER}" = "NONE" ] ; then
		SC_USER=""
	else
		grep -q "^${SC_USER}" ${MY_DATA}/.htpassword
		if [ $? = 0 ] ; then
			echo "User ${SC_USER} is already defined ... ADD folder to it ... if folder name is different"
			grep -q "^${FOLDER_NAME}" ${MY_DATA}/folder.conf
			if [ $? = 0 ] ; then
				echo "Hum! folder is already defined ????"
				exit 1
			fi
		else
			echo "Need to create a new user : ${SC_USER} with password or asusfr1 as default in this case CHANGE IT"
			if [ ! -z "${P}" ] ; then
				if [ "${P}" = "${PC}" ] ; then
					echo -e "Password provide and confirmed"
					PASS="${P}"
				else
					echo -e "Use default password "
					PASS="asusfr1"
				fi
			else
				# no password provide use asusfr1
				PASS="asusfr1"
			fi
			$0 add_new_user_password ${SC_USER} ${PASS} ${PASS}
		fi
	fi
	
	if [ -e "${PATH_TO_FOLDER}" ] ; then
		echo -e "Create new folder in folder.conf"
		create_new_folder
		echo -e "Link json file from folder to script-server runners"
		create_link_for_folder
	else
		echo "PATH ${PATH_TO_FOLDER} do not exist OR don't have a script_server folder inside  ? "
		### perhaps delete fuser created
		exit 1
	fi
;;
delete_folder)
	FOLDER_NAME="${2}"
	if  [ -z "${FOLDER_NAME}" ] ; then
		echo "missing parameters folder_name"
		exit 1
	fi
	grep ^"${FOLDER_NAME}": ${MY_DATA}/folder.conf
	if [ $? -ne 0 ] ; then
		echo "${FOLDER_NAME} don't exist in folder.conf ??? do a list to  verify"
		exit 1
	fi
	delete_folder
;;
use_apkg_htpasswd)
	REP="${2}"
	if [ "${REP}" = "YES" -o "${REP}" = "yes" ] ; then
		echo "use htpasswd provide by the APKG"
		if [ ! -e ${MY_DATA}/.use_my_htpasswd ] ; then
			touch ${MY_DATA}/.use_my_htpasswd
		fi
	else
		echo -e "use htpasswd from Asustor http APKG if exist or revert to internal one"
		rm -f ${MY_DATA}/.use_my_htpasswd
	fi
;;
*)
	echo -e "Usage : (${RED}RED${NC} is for \"${RED}dangerous\"${NC}) "
	echo -e "$(basename $0) ${BLUE}start|stop|restart|clean_and_restart${NC}"
	echo -e "$(basename $0) ${BLUE}status${NC}"
	echo -e "$(basename $0) ${BLUE}relink_admin_json${NC} ... relink admin json files"
	echo -e "$(basename $0) ${BLUE}integrate_apkg_folder [PATH_to_tgz_file]${NC} ...\n                ADD script-server script for a specific folder or APKG (read Doc for detail)"
	echo "====="
	echo -e "$(basename $0) ${BLUE}webui_port_used${NC} ... by web script_server server"
	echo -e "$(basename $0) ${RED}change_webui_port_used PORT${NC} ...\n                Change in AppCentral (icon) and in script-server server config file ${RED}THEN RESTART${NC}"
	echo -e "$(basename $0) ${RED}change_to_https${NC} ... \n                Change to use HTTPS (if http is used) DO or require a modification in server json file (read the doc) ${RED}THEN RESTART${NC}"
	echo -e "$(basename $0) ${RED}change_to_http${NC} ... \n                Change to use HTTP (if https is used) DO or require a modification in server json file (read the doc) ${RED}THEN RESTART${NC}"
	echo "====="
	echo -e "$(basename $0) ${BLUE}use_apkg_htpasswd [YES|NO]${NC} ... use htpasswd provide in this APKG or from Asustor Apache (if web server installed)"
	echo "====="
	echo -e "$(basename $0) ${BLUE}save_settings${NC} ... save conf folder in /share/Public"
	echo -e "$(basename $0) ${BLUE}save_all_logs${NC} ... save logs folder in /share/Public"
	echo -e "$(basename $0) ${BLUE}remove_history_logs [Script-Server_USER_NAME]${NC} ... \n                   remove history of shell executed for ALL or for a specific USER only"
	echo "====="
	echo -e "$(basename $0) ${BLUE}list_users_defined${NC} ... list users define to access WebUI"
	echo -e "$(basename $0) ${BLUE}verif_user_password EXISTING_USER_NAME [Current_Password]${NC} ... \n               verify the actual password for the user ... providing password in command is less secure"
	echo -e "$(basename $0) ${BLUE}add_new_user_password NEW_USER_NAME [Password Confirm_Password]${NC} ... \n               add WebUI access user/password ... providing password in command is less secure"
	echo -e "$(basename $0) ${BLUE}change_user_password EXISTING_USER_NAME [Password Confirm_Password]${NC} ... \n               change WebUI password for existing user ... providing password in command is less secure"
	echo -e "$(basename $0) ${BLUE}supress_user_password EXISTING_USER_NAME${NC} ... \n               suppress existing user from accessing WebUI"
	echo "====="
	echo -e "$(basename $0) ${BLUE}list_folder_defined${NC} ... \n               list APKG/FOLDER with script-server interface defined"
	echo -e "$(basename $0) ${BLUE}create_new_folder FOLDER_NAME FOLDER_PATH {USER_NAME [PASSWORD CONFIRM_PASSWORD] | NONE} ${NC} ... \n               create a new folder with script NO password we use asusfr1"
	echo -e "$(basename $0) ${BLUE}create_folder_link [FOLDER_TARGET]${NC} ... \n               create script link for an APKG/FOLDER on specific script-server user"
	echo -e "$(basename $0) ${BLUE}delete_folder FOLDER_NAME${NC} ... \n               suppress json link & folder in folder.conf NOT original json & script"
	echo "====="
;;
esac

